#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import json
import os
import os.path
import platform
import shlex
import subprocess
import urllib
from urllib.parse import urljoin

import requests

from TestToolsMK.robot_instances import osl
import time

from robot.api import logger
from robot.api.deco import keyword, library

import base64
from io import BytesIO


def get_base64(image, format="JPEG"):
    buffered = BytesIO()
    image.save(buffered, format=format)
    img_str = base64.b64encode(buffered.getvalue())
    return "data:image/jpeg;base64," + img_str.decode()


def sizeof_fmt(num, suffix="B"):
    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, "Yi", suffix)


def wait_net_service(server, port, timeout=None):
    """Wait for network service to appear
    @param port:
    @param server:
    @param timeout: in seconds, if None or 0 wait forever
    @return: True of False, if timeout is None may return only True or
             throw unhandled network exception
    """
    # noinspection PyGlobalUndefined,PyGlobalUndefined
    global end, now
    import socket
    import errno

    s = socket.socket()
    if timeout:
        from time import time as now

        # time module is needed to calc timeout shared between two exceptions
        end = now() + timeout

    while True:
        try:
            if timeout:
                next_timeout = end - now()
                if next_timeout < 0:
                    return False
                else:
                    s.settimeout(next_timeout)

            s.connect((server, port))

        except socket.timeout as err:
            # this exception occurs only if timeout is set
            if timeout:
                return False

        except socket.error as err:
            # catch timeout exception from underlying network library
            # this one is different from socket.timeout
            if type(err.args) != tuple or err[0] != errno.ETIMEDOUT:
                raise

        except:
            # catch timeout exception from underlying network library
            # this one is different from socket.timeout
            if timeout:
                return False
        else:
            s.close()
            return True


@library
class UtilsKeywords(object):
    @property
    def get_latest_chrome_driver_version():
        return (
            urllib.urlopen("http://chromedriver.storage.googleapis.com/LATEST_RELEASE")
            .read()
            .strip()
        )

    @property
    def get_latest_firefox_driver_version():
        json_raw = (
            urllib.urlopen(
                "https://api.github.com/repos/mozilla/geckodriver/releases/latest"
            )
            .read()
            .strip()
        )
        json_data = json.loads(json_raw)
        return json_data["tag_name"].strip()

    @property
    def get_url_for_latest_chrome_driver():
        last_build = self.get_latest_chrome_driver_version
        try:
            base_url_for_driver = (
                "http://chromedriver.storage.googleapis.com/" + last_build + "/"
            )
            if platform.system() == "Windows":
                return base_url_for_driver + "chromedriver_win32.zip"
            if platform.system() == "Linux":
                return base_url_for_driver + "chromedriver_linux64.zip"

        except os.error as e:
            logger.warn("Unexpected error" + e)
            return "missing"

    @property
    def get_url_for_latest_firefox_driver():
        last_build = self.get_latest_firefox_driver_version
        try:
            base_url_for_driver = (
                "https://github.com/mozilla/geckodriver/releases/download/"
                + last_build
                + "/"
            )
            if platform.system() == "Windows":
                return base_url_for_driver + "geckodriver-" + last_build + "-win64.zip"
            if platform.system() == "Linux":
                return (
                    base_url_for_driver
                    + "geckodriver-"
                    + last_build
                    + "-linux64.tar.gz"
                )

        except os.error as e:
            logger.warn("Unexpected error" + e)
            return "missing"

    @keyword
    def get_chrome_driver_latest(self, path="./bin"):
        """
        Download Latest Chrome Driver and add it to system path (only for this session).
        If system already contains proper version do nothing.
        Based on information from http://chromedriver.storage.googleapis.com/LATEST_RELEASE
        """
        try:
            initial = osl().get_environment_variable("PATH")
            path_abstract = os.path.abspath(path)

            if path_abstract not in initial:
                osl().set_environment_variable(
                    "PATH", path_abstract + os.pathsep + initial
                )

            try:
                version = subprocess.check_output(
                    ["chromedriver", "--version"], shell=True, stderr=subprocess.STDOUT
                )
            except subprocess.CalledProcessError as e:
                version = "Chrome driver is MISSING "

            version_current = version.strip()
            version_latest = self.get_latest_chrome_driver_version
            url_chrome = self.get_url_for_latest_chrome_driver
            logger.debug("Current version \t" + version_current)
            logger.debug("Latest version :\t" + version_latest)
            if version_latest not in version_current:
                logger.info(
                    "Current version %s , latest is %s"
                    % (version_current, version_latest)
                )
                if not os.path.exists(path_abstract):
                    os.makedirs(path_abstract)
                logger.info("start download chrome driver :" + url_chrome)

                driver = urllib.urlopen(url_chrome).read()
                logger.debug(
                    "Chrome driver compressed size: " + sizeof_fmt(len(driver))
                )

                from zipfile import ZipFile
                from io import BytesIO

                with ZipFile(BytesIO(driver)) as zfile:
                    zfile.extractall(path_abstract)
                logger.info("Chrome driver extracted to : " + path_abstract)

            else:
                logger.info(
                    "Latest Version of ChromeDriver Present %s , latest %s"
                    % (version_current, version_latest)
                )

        except KeyError as e:
            logger.error(e)
        except OSError as e:
            logger.error(e)

    @keyword
    def get_firefox_driver_latest(self, path="./bin"):
        """
        Download Latest Firefox geckodriver Driver and add it to system path (only for this session).
        If system already contains proper version do nothing.
        Based on information from https://api.github.com/repos/mozilla/geckodriver/releases/latest
        """

        try:
            initial = osl().get_environment_variable("PATH")
            path_abstract = os.path.abspath(path)

            if path_abstract not in initial:
                osl().set_environment_variable(
                    "PATH", path_abstract + os.pathsep + initial
                )

            try:
                version = subprocess.check_output(
                    ["geckodriver", "--version"], shell=True, stderr=subprocess.STDOUT
                )
            except subprocess.CalledProcessError as e:
                version = "Firefox geckodriver driver is MISSING "

            version_current = version.strip()
            version_latest = self.get_latest_firefox_driver_version.replace(
                "v", ""
            ).strip()
            url_firefox = self.get_url_for_latest_firefox_driver
            logger.debug("Current version \t" + version_current)
            logger.debug("Latest version :\t" + version_latest)
            if version_latest not in version_current:
                logger.info(
                    "Current version %s , latest is %s"
                    % (version_current, version_latest)
                )

                logger.info("start download firefox geckodriver driver :" + url_firefox)
                if not os.path.exists(path_abstract):
                    os.makedirs(path_abstract)
                driver = urllib.urlopen(url_firefox).read()
                logger.debug(
                    "Firefox geckodriver driver compressed size: "
                    + sizeof_fmt(len(driver))
                )

                from zipfile import ZipFile
                from io import BytesIO

                with ZipFile(BytesIO(driver)) as zfile:
                    zfile.extractall(path_abstract)
                logger.info("Chrome driver extracted to : " + path_abstract)

            else:
                logger.info(
                    "Latest Version of Firefox geckodriver Present %s , latest %s"
                    % (version_current, version_latest)
                )

        except KeyError as e:
            logger.error(e)
        except OSError as e:
            logger.error(e)

    @keyword
    def get_selenium_server(
        url="https://goo.gl/lbAQcq",
        path="./bin/selenium-server.jar",
        skip_if_already_exists="True",
    ):
        """
        Currently hard coded as arg future change to download from https://selenium-release.storage.googleapis.com
        """
        dir_selenium = os.path.dirname(path)
        path = os.path.abspath(path)
        logger.debug("Folder to download " + dir_selenium)

        if not os.path.exists(dir_selenium):
            os.makedirs(dir_selenium)

        if skip_if_already_exists == "True":
            if os.path.exists(path):
                logger.info(
                    'Skip Selenium Server download already exists in path "%s" because skip_if_already_exists is set to "%s"'
                    % (path, skip_if_already_exists)
                )
                return

        try:
            r = requests.head(url, allow_redirects=True)
            logger.info("Resolved url: \t" + r.url)
            testfile = urllib.URLopener()
            testfile.retrieve(r.url, path)
            logger.info("Selenium Server download completed to " + path)

        except KeyError as e:
            logger.error(e)
        except OSError as e:
            logger.error(e)

    @keyword
    def start_selenium_server(
        self,
        path="./bin/selenium-server.jar",
        timeout=60,
        port="4444",
        logs_path="./bin",
    ):
        """
        :return:
        """
        path = os.path.abspath(path)

        with open(
            os.path.abspath(logs_path + "/selenium_server_stdout.txt"), "wb"
        ) as out, open(
            os.path.abspath(logs_path + "/selenium_server_stderr.txt"), "wb"
        ) as err:
            command = 'java -jar "' + path + '" -port ' + port
            logger.info("Command to start server :" + command)
            # noinspection PyAttributeOutsideInit
            self.selenium_server = subprocess.Popen(
                shlex.split(command), stdout=out, stderr=err
            )
        time.sleep(7)

    @keyword
    def shutdown_selenium_server(self):
        if self.selenium_server is not None:
            self.selenium_server.terminate()
            logger.info("Selenium Server shutdown")
        else:
            logger.error("Server not started")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski


from robot.api.deco import keyword, library


@library
class CollectionsKeywords(object):
    @keyword
    def create_dictionary_from_list(self, table):
        return dict((x, 0) for x in table)

    @keyword
    def create_dictionary_from_two_lists(self, keys, values):
        return dict(zip(keys, values))

    @keyword
    def sort_list_by_number(self, list):
        """Sorts the given list in place."""
        list.sort(key=float)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski

import os
import os.path


from robot.api import logger

from PIL import Image, ImageChops, ImageStat, ImageOps

from robot.api.deco import keyword, library

from urllib.parse import urljoin
from urllib.request import pathname2url


@library
class ImagePillowKeywords(object):
    def _difference_image_percent(self, i1, i2):
        pairs = zip(i1.getdata(), i2.getdata())
        if len(i1.getbands()) == 1:
            # for gray-scale jpegs
            dif = sum(abs(p1 - p2) for p1, p2 in pairs)
        else:
            dif = sum(abs(c1 - c2) for p1, p2 in pairs for c1, c2 in zip(p1, p2))

        ncomponents = i1.size[0] * i1.size[1] * 3
        temp = (dif / 255.0 * 100) / ncomponents
        return temp

    def _compare_image_files(
        self,
        file_1_path,
        file_2_path,
        gif_file_path=None,
        delta_file_path=None,
        embedded_gif=True,
        embedded_delta=False,
    ):
        file_1 = os.path.normpath(file_1_path)
        file_2 = os.path.normpath(file_2_path)

        if delta_file_path is not None:
            delta_file = os.path.normpath(delta_file_path)
        else:
            delta_file_path = (
                "./"
                + os.path.splitext(os.path.basename(file_1))[0]
                + "_"
                + os.path.splitext(os.path.basename(file_2))[0]
                + "_delta.png"
            )
            delta_file = os.path.normpath(delta_file_path)

        if gif_file_path is not None:
            gif_file = os.path.normpath(gif_file_path)
        else:
            gif_file_path = (
                "./"
                + os.path.splitext(os.path.basename(file_1))[0]
                + "_"
                + os.path.splitext(os.path.basename(file_2))[0]
                + ".gif"
            )
            gif_file = os.path.normpath(gif_file_path)
        if os.path.isfile(file_1) and os.path.isfile(file_2):

            logger.debug("Opening file" + file_1)
            img1 = Image.open(file_1)
            logger.debug("Opening file" + file_2)

            img2 = Image.open(file_2)

            # finding difference
            diff = ImageChops.difference(img1, img2)

            diff = diff.convert("RGB")

            diff.save(delta_file)

            delta_percent = self._difference_image_percent(img1, img2)

            if gif_file_path is not None:
                self.create_gif_from_three_files(
                    gif_file,
                    file_1,
                    file_2,
                    delta_file,
                    embedded=embedded_gif,
                )
            if delta_file_path is None:
                os.remove(delta_file)
            else:
                if embedded_delta:
                    self._embed_screenshot(delta_file)

            return (
                delta_percent,
                delta_file,
                gif_file,
            )

    @keyword
    def compare_image_files(
        self,
        file_1_path,
        file_2_path,
        gif_file_path=None,
        delta_file_path=None,
        embedded_gif=True,
        embedded_delta=False,
    ):
        results = self._compare_image_files(
            file_1_path,
            file_2_path,
            gif_file_path,
            delta_file_path,
            embedded_gif,
            embedded_delta,
        )

        actual_percent = results[0]
        return actual_percent

    @keyword
    def image_should_be_difference_less_then(
        self,
        file_1_path,
        file_2_path,
        difference_percent=1,
        gif_file_path=None,
        delta_file_path=None,
        embedded_gif=True,
        embedded_delta=False,
    ):
        """difference_percent test to 0 mean both images are identical"""

        results = self._compare_image_files(
            file_1_path,
            file_2_path,
            gif_file_path,
            delta_file_path,
            embedded_gif=embedded_gif,
            embedded_delta=embedded_delta,
        )
        actual_percent = results[0]
        if actual_percent > difference_percent:
            message = f"Difference between files is greater then expected actual {actual_percent:.2f} > {difference_percent:.2f} expected percent"

            raise AssertionError(message)
        else:
            logger.info("Image check successful")
        return actual_percent

    @keyword
    def create_gif_from_three_files(
        self,
        gif_file_path,
        file_1_path,
        file_2_path,
        file_3_path,
        delay_ms=500,
        loop=0,
        embedded=True,
    ):
        files_list = [file_1_path, file_2_path, file_3_path]
        self.create_gif_from_list_of_files(
            gif_file_path, files_list, delay_ms, loop, embedded
        )

    @keyword
    def create_gif_from_list_of_files(
        self, gif_file_path, list_of_files, delay_ms=500, loop=0, embedded=True
    ):
        images = []
        for file in list_of_files:
            temp = Image.open(file)
            images.append(temp)

        images[0].save(
            gif_file_path,
            save_all=True,
            append_images=images,
            duration=delay_ms,
            loop=loop,
        )
        if embedded:
            self._embed_screenshot(gif_file_path)

    @staticmethod
    def _embed_screenshot(path, level="INFO", width="1200px"):
        link = urljoin("file:", pathname2url(os.path.normpath(path)))
        logger.write(
            '</td></tr><tr><td colspan="3"><a href="%s"><img src="%s" width="%s"></a>'
            % (link, link, width),
            level,
            html=True,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import os

from TestToolsMK.collections_keywords import CollectionsKeywords
from TestToolsMK.csv_keywords import CsvKeywords
from TestToolsMK.image_pillow_keywords import ImagePillowKeywords
from TestToolsMK.logger_extension_keywords import LoggerKeywords
from TestToolsMK.selenium_extensions_keywords import ExtendedSeleniumLibrary
from TestToolsMK.sql_keywords import SQLKeywords
from TestToolsMK.timers_keywords import TimerKeywords
from TestToolsMK.utils import UtilsKeywords

__version_file_path__ = os.path.join(os.path.dirname(__file__), "VERSION")
__version__ = open(__version_file_path__, "r").read().strip()


class TestToolsMK(
    CollectionsKeywords,
    CsvKeywords,
    ImagePillowKeywords,
    LoggerKeywords,
    ExtendedSeleniumLibrary,
    SQLKeywords,
    TimerKeywords,
    UtilsKeywords,
):
    ROBOT_LIBRARY_SCOPE = "GLOBAL"

    def __init__(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import os


import string
import random
from datetime import datetime
import codecs

from robot.api import logger
from robot.libraries.DateTime import Date
from robot.libraries.DateTime import Time
from robot.utils import is_falsy

from TestToolsMK.robot_instances import dbl, ttmkl, validate_create_artifacts_dir
import sys


def get_current_time_for_timers():
    return datetime.now()


# noinspection PyProtectedMember
from robot.api.deco import keyword, library


# noinspection PyProtectedMember
@library
class SQLKeywords(object):
    OUTPUT_FILE_LOG_SQL = "Artifacts/log_of_sql_execution.sql"
    ADD_LOGS_FLAG = True

    @keyword
    def set_sql_log_output_file(self, path="Artifacts/log_of_sql_execution.sql"):
        self.OUTPUT_FILE_LOG_SQL = validate_create_artifacts_dir(path)

    @keyword
    def set_add_logs_flag(self, flag=False):
        self.ADD_LOGS_FLAG = flag

    @keyword
    def query_many_rows(self, select_statement, append_to_logs=ADD_LOGS_FLAG):
        """
        To switch output file with logs use
        | Set Sql Log Output File | ./myFile.sql |
        :param select_statement:
        :param append_to_logs:
        :return:
        """
        if append_to_logs:
            self._add_query_to_log_file(select_statement)
        results = dbl().query(select_statement)
        if append_to_logs:
            self._add_results_to_log_file(results)
        return results

    @keyword
    def query_row(self, select_statement, append_to_logs=ADD_LOGS_FLAG):
        """
        To switch output file with logs use
        | Set Sql Log Output File | ./myFile.sql |
        :raise error when results contains more then one row results
        :return: table
        """
        results = self.query_many_rows(select_statement, append_to_logs)

        if len(results) > 1:
            message = (
                "Error. Results contains more then one row. Actual size is %s "
                % (len(results))
            )
            self._append_to_file("/* \n" + message + "\n */")
            raise AssertionError(message)

        return results[0]

    @keyword
    def query_cell(self, select_statement, append_to_logs=ADD_LOGS_FLAG):
        """
        To switch output file with logs use
        | Set Sql Log Output File | ./myFile.sql |
        :raise error when results contains more then one cell
        :return: single value
        """
        results = self.query_row(select_statement, append_to_logs)
        if len(results) > 1:
            message = (
                "Error. Results contains more then one cell. Actual size is %s "
                % (len(results))
            )
            self._append_to_file("/* \n" + message + "\n */")
            raise AssertionError(message)
        return results[0]

    def _add_query_to_log_file(self, statement):
        current_time = get_current_time_for_timers()
        self.start_time = current_time
        final_string = (
            "\n/* Start execution, statement below : "
            + current_time.strftime("%Y.%m.%d %H:%M:%S")
            + " */\n"
            + statement
            + "\n"
        )
        self._append_to_file(final_string)


    def _add_results_to_log_file(self, results):
        current_time = get_current_time_for_timers()
        total_time_verbose = Time(current_time - self.start_time).convert("verbose")

        if results:
            data = "[\n"
            for i in range(len(results)):
                data += "("
                for j in range(len(results[i])):
                    data += repr(results[i][j])
                    if j != (len(results[i]) - 1):
                        data += "\t,"
                data += ")\n"
            data += "]"
        else:
            data = ""
        final_string = (
            "/* Response of statement in :"
            + total_time_verbose
            + " , data below "
            + data
            + " */\n"
        )
        self._append_to_file(final_string)

    def _append_to_file(self, text):
        full_log_file_path = validate_create_artifacts_dir(self.OUTPUT_FILE_LOG_SQL)
        modeFile = "a" if os.path.exists(full_log_file_path) else "w"
        with codecs.open(full_log_file_path, modeFile, "utf-8") as output:
            output.write(text)

    @keyword
    def execute_sql_string_with_logs(self, sql_string, append_to_logs=ADD_LOGS_FLAG):
        """
        To switch output file with logs use
        | Set Sql Log Output File | ./myFile.sql |
        :param sql_string:
        :param append_to_logs:
        :return:
        """
        if append_to_logs:
            self._add_query_to_log_file(sql_string)
        dbl().execute_sql_string(sql_string)
        if append_to_logs:
            self._add_results_to_log_file(None)

    @keyword
    def insert_data_to_table(self, table_name, data):
        """
        return table name, table will have columns with names 'c0', 'c1' for all columns
        table will be also reindex after insert
        """
        cur = dbl()._dbconnection.cursor()

        size = len(data)
        if size < 1:
            raise AssertionError("missing data 0 rows")

        row_size = len(data[0])
        if row_size < 1:
            raise AssertionError("missing data 0 columns")

        columns_desc = ""
        for index in range(len(data[0])):
            columns_desc += """"c""" + str(index) + """" VARCHAR"""
            if index < (row_size - 1):
                columns_desc += ","

        create_sql = 'CREATE TABLE "%s" (%s)' % (table_name, columns_desc)

        logger.info(create_sql)
        cur.execute(create_sql)

        columns_names = ""
        values = ""
        for index in range(len(data[0])):
            columns_names += "c" + str(index)
            values += """?"""
            if index < (row_size - 1):
                columns_names += ", "
                values += ", "

        insert_data_sql = 'INSERT INTO "main"."%s" (%s) VALUES (%s) ' % (
            table_name,
            columns_names,
            values,
        )

        logger.info(insert_data_sql)
        cur.executemany(insert_data_sql, data)

        reindex_sql = 'REINDEX "main"."%s"' % table_name
        cur.execute(reindex_sql)

        dbl()._dbconnection.commit()

        return

    @keyword
    def insert_data_to_generated_table(self, data):
        table_name = table_name_generator()
        self.insert_data_to_table(table_name, data)
        return table_name

    @keyword
    def csv_read_file_to_database(
        self, table_name, csv_file, encoding="UTF-8", encoding_errors="strict"
    ):
        """Use only for temporary use
        connect to database in memory if not connected, create table with name and import data from csv file"""

        if dbl()._dbconnection is None:
            dbl().connect_to_database_using_custom_params("sqlite3", "':memory:'")
        array_table = ttmkl().csv_read_file(csv_file, encoding, encoding_errors)
        self.insert_data_to_table(table_name, array_table)
        return table_name

    @keyword
    def connect_to_database_using_jdbc_driver(
        self,
        jdbc_connection_string,
        user,
        password,
        jdbc_driver,
        jdbc_jar_path,
        library="jaydebeapi",
    ):
        """
        Example parameters for CIS databases Cisto Integration Services, using java drivers is done using library
        jdbc_connection_string: jdbc:compositesw:dbapi@localhost:9401?domain=composite&dataSource=Example
        jdbc_driver: cs.jdbc.driver.CompositeDriver
        jdbc_jar_path: Binaries/csjdbc.jar
        jaydebeapi library has to be installed

        """
        dbl().connect_to_database_using_custom_params(
            library,
            '"%s","%s",[ "%s", "%s"],"%s",'
            % (jdbc_driver, jdbc_connection_string, user, password, jdbc_jar_path),
        )


def table_name_generator(
    size=12, chars=string.ascii_lowercase + string.ascii_uppercase
):
    return "".join(random.choice(chars) for _ in range(size))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
from mimetypes import init
import os
import os.path
import time

from robot.api import logger
from robot.libraries import DateTime
from selenium.webdriver import FirefoxProfile, ChromeOptions
from selenium.webdriver.common.keys import Keys

from TestToolsMK.robot_instances import validate_create_artifacts_dir, sl, bi

from robot.api.deco import keyword, library


# noinspection PyProtectedMember
@library
class ExtendedSeleniumLibrary(object):
    WIDTH_DEFAULT = "1366"
    HEIGHT_DEFAULT = "768"
    SELENIUM_SPEED = "0 sec"
    SELENIUM_TEST_BROWSER = "ff"
    SELENIUM_TIMEOUT = "5 s"
    # noinspection PyPep8
    XPATH2_JS = "if(!window.jQuery){var headID = document.getElementsByTagName(\"head\")[0]; var newScript = document.createElement('script'); newScript.type='text/javascript'; newScript.src='http://llamalab.com/js/xpath/minified/XPath.js'; headID.appendChild(newScript);}"
    # noinspection PyPep8
    JQUERY_JS = "if(!window.jQuery){var headID = document.getElementsByTagName(\"head\")[0]; var newScript = document.createElement('script'); newScript.type='text/javascript'; newScript.src='http://code.jquery.com/jquery-2.1.4.min.js'; headID.appendChild(newScript);}"
    SELENIUM_LIBRARY = None

    def __init__(self):
        self.SELENIUM_LIBRARY = sl()

    @keyword
    def open_new_tab(self, url):
        """Hack it use Control +t to open new tab"""
        driver = sl().driver
        body = driver.find_element_by_tag_name("body")
        body.send_keys(Keys.CONTROL + "t")
        time.sleep(2)
        sl().go_to(url)

    @keyword
    def switch_tab_by_id(self, id_tab):
        """Hack it use Control + 1,2,3 etc to switch tab"""
        driver = sl().driver
        body = driver.find_element_by_tag_name("body")
        body.send_keys(Keys.CONTROL + id_tab)
        time.sleep(4)
        # actions = ActionChains(driver)
        # actions.key_down(Keys.CONTROL).key_down(Keys.TAB).key_up(Keys.TAB).key_up(Keys.CONTROL).perform()

    @keyword
    def press_key_python(self, command, locator="//body", strategy="XPATH"):
        """Hack !!!  example argument | Keys.CONTROL + 't' |Keys.TAB + Keys.SHIFT"""
        driver = sl().driver
        element = driver.find_element(eval("By." + strategy), locator)
        element.send_keys(eval(command))

    @keyword
    def close_tab(self):
        """Hack it use Control +w to close tab"""
        driver = sl().driver
        body = driver.find_element_by_tag_name("body")
        body.send_keys(Keys.CONTROL + "w")

    @keyword
    def set_browser_size_and_position(
        self, width=WIDTH_DEFAULT, height=HEIGHT_DEFAULT, x=0, y=0
    ):
        sl().set_window_size(width, height)
        sl().set_window_position(x, y)

    @keyword
    def go_to_smart(self, url):
        """Redirect only in on different url"""
        current_url = sl().get_location()
        if url != current_url:
            sl().go_to(url)

    @keyword
    def click_element_extended(
        self, locator, modifier=False, action_chain=False, timeout=None, error_msg=None
    ):
        """
        Click element proceed with following steps
        * wait_until_page_contains_element
        * wait_until_element_is_visible_wait_until_element_is_visible
        * scroll_element_into_view
        * mouse_over
        * click_element

        """
        sl().wait_until_page_contains_element(locator, timeout, error_msg)
        sl().wait_until_element_is_visible(locator, timeout, error_msg)
        sl().scroll_element_into_view(locator)
        sl().mouse_over(locator)
        sl().click_element(locator, modifier=modifier, action_chain=action_chain)

    @keyword
    def double_click_element_extended(
        self, locator, modifier=False, action_chain=False, timeout=None, error=None
    ):
        """
        Double Click element proceed with following steps
        * wait_until_page_contains_element
        * wait_until_element_is_visible_wait_until_element_is_visible
        * scroll_element_into_view
        * mouse_over
        * double_click_element
        """
        sl().wait_until_page_contains_element(locator, timeout, error)
        sl().wait_until_element_is_visible(locator, timeout, error)
        sl().scroll_element_into_view(locator)
        sl().mouse_over(locator)
        sl().double_click_element(locator, modifier=modifier, action_chain=action_chain)

    @keyword
    def click_element_extended_and_wait(
        self,
        locator,
        sleep,
        modifier=False,
        action_chain=False,
        timeout=None,
        error_msg=None,
        reason=None,
    ):
        sl().click_element_extended(locator, timeout, error_msg)
        bi().sleep(sleep, reason)

    @keyword
    def open_browser_extension(
        self,
        url,
        browser="ff",
        width=WIDTH_DEFAULT,
        height=HEIGHT_DEFAULT,
        x="0",
        y="0",
        alias=None,
        remote_url=False,
        desired_capabilities=None,
        ff_profile_dir=None,
        selenium_timeout=SELENIUM_TIMEOUT,
        keyword_to_run_on_failure="Capture Page Screenshot Extension",
    ):
        sl().open_browser(
            "about:blank",
            browser,
            alias,
            remote_url,
            desired_capabilities,
            ff_profile_dir,
        )
        sl().set_window_position(x, y)
        sl().set_window_size(width, height)
        sl().set_selenium_timeout(selenium_timeout)
        sl().register_keyword_to_run_on_failure(keyword_to_run_on_failure)
        sl().go_to(url)

    @keyword
    def import_xpath2(self):
        sl().execute_javascript(self.XPATH2_JS)

    # noinspection PyPep8Naming,PyPep8Naming
    @keyword
    def import_jQuery(self):
        sl().execute_javascript(self.JQUERY_JS)

    # noinspection PyProtectedMember
    @keyword
    def capture_page_screenshot_extension(
        self,
        prefix="",
        postfix="",
        add_time_stamp=True,
        add_test_case_name=True,
        add_file_path_to_list="${list of screenshots}",
        output_dir="Artifacts/Screenshots",
    ):
        output_dir_normalized = validate_create_artifacts_dir(output_dir)

        if add_time_stamp:
            current_time = " " + DateTime.get_current_date(
                result_format="%Y.%m.%d_%H.%M.%S"
            )
        else:
            current_time = ""
        if add_test_case_name:
            test_case_name = bi().get_variable_value("${TEST_NAME}")
        else:
            test_case_name = ""

        output_file = (
            output_dir_normalized
            + "/"
            + prefix
            + test_case_name
            + postfix
            + current_time
            + ".png"
        )
        output_file_normalized = os.path.normpath(output_file)

        # sl()driver.get_screenshot_as_file(output_file_normalized)
        sl().capture_page_screenshot(output_file_normalized)

        results = bi().run_keyword_and_return_status(
            "Variable Should Exist", add_file_path_to_list
        )

        if not results:
            bi()._get_var_name(add_file_path_to_list)
            list_with_files = bi().create_list(output_file_normalized)
            bi().set_test_variable(add_file_path_to_list, list_with_files)
        else:
            list_with_files = bi().create_list(output_file_normalized)
            list_with_files = bi().run_keyword(
                "Combine Lists", add_file_path_to_list, list_with_files
            )
            bi().set_test_variable(add_file_path_to_list, list_with_files)

        return output_file_normalized

    @keyword
    def element_attribute_should_be(
        self, locator, attribute, attribute_value_expected, msg=None, values=True
    ):
        actual_value = sl().get_element_attribute(locator + "@" + attribute)
        # noinspection PyProtectedMember
        actual_value, attribute_value_expected = [
            bi()._convert_to_string(i) for i in (actual_value, attribute_value_expected)
        ]
        bi()._should_be_equal(actual_value, attribute_value_expected, msg, values)

    # noinspection SpellCheckingInspection,SpellCheckingInspection,SpellCheckingInspection,SpellCheckingInspection
    @keyword
    def create_download_dir_profile_for_firefox(
        self, path_to_download, mime_types_file=None, *extensions_files
    ):
        """
        Example use
        | ${profile} | Create Download Dir Profile For Firefox | Artifacts | Resources/mimeTypes.rdf | Resources/webdriver_element_locator-2.0-fx.xpi | Resources/selenium_ide-2.9.1-fx.xpi |
        | Open Browser Extension | https://support.spatialkey.com/spatialkey-sample-csv-data/ | ff_profile_dir=${profile} |
        | Click Element | //a[contains(@href,'sample.csv.zip')]  |
        """
        path_to_download_check = validate_create_artifacts_dir(path_to_download)

        fp = FirefoxProfile()
        fp.set_preference("browser.download.folderList", 2)
        fp.set_preference("browser.download.manager.showWhenStarting", False)
        fp.set_preference("browser.download.manager.alertOnEXEOpen", False)
        fp.set_preference("browser.download.dir", path_to_download_check)
        fp.set_preference("xpinstall.signatures.required", False)
        fp.set_preference("browser.helperApps.alwaysAsk.force", False)
        fp.set_preference(
            "browser.helperApps.neverAsk.saveToDisk",
            "application/msword;application/csv;text/csv;image/png;image/jpeg;application/pdf;text/html;text/plain;application/octet-stream",
        )
        fp.set_preference("pdfjs.disabled", True)
        fp.update_preferences()
        for single_extension in extensions_files:
            fp.add_extension(single_extension)
        if mime_types_file is not None:
            mime_types_file = os.path.abspath(mime_types_file)
            from shutil import copy2

            copy2(os.path.normpath(mime_types_file), fp.profile_dir)
        logger.info("Firefox Profile Created in dir '" + fp.profile_dir + "'")
        return fp.profile_dir

    # noinspection SpellCheckingInspection,SpellCheckingInspection
    @keyword
    def create_download_dir_capabilities_for_chrome(
        self, path_to_download, **extensions_files
    ):
        """
        Example use
        | ${capabilities} |	create_download_dir_capabilities_for_chrome	| Artifacts |
        | Open Browser Extension | https://support.spatialkey.com/spatialkey-sample-csv-data/ |	gc | desired_capabilities=${capabilities} |
        | Click Element	 | //a[contains(@href,'sample.csv.zip')] |
        """

        path_to_download_check = validate_create_artifacts_dir(path_to_download)

        chrome_options = ChromeOptions()
        prefs = {
            "download.default_directory": path_to_download_check,
            "directory_upgrade": "true",
        }

        chrome_options.add_experimental_option("prefs", prefs)
        chrome_options.add_argument("--disable-web-security")
        for single_extension in extensions_files:
            chrome_options.add_extension(single_extension)

        logger.info(
            "Chrome Capabilities set download dir '" + path_to_download_check + "'"
        )
        return chrome_options.to_capabilities()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  Copyright (c) 2015 Cutting Edge QA Marcin Koperski


"""Setup script for Robot's Marcin Koperski bundle distributions"""

import os
import sys

from setuptools import setup

src_path = os.path.join(os.path.dirname(__file__), "src")
sys.path.insert(0, src_path)

__version_file_path__ = os.path.join(src_path, "TestToolsMK", "VERSION")
__version__ = open(__version_file_path__, "r").read().strip()

setup(
    name="robotframework-MarcinKoperski",
    version=__version__,
    description="RobotFramework Marcin Koperski bundle",
    author="Marcin Koperski",
    license="MIT",
    author_email="marcin.koperski+pypi@gmail.com",
    url="https://github.com/IlfirinPL/robotframework-MarcinKoperski",
    download_url="https://github.com/IlfirinPL/robotframework-MarcinKoperski/archive/master.zip".format(
        version=__version__
    ),
    keywords=["robotframework", "MarcinKoperski"],
    package_dir={"": "src"},
    packages=["TestToolsMK"],
    exclude_package_data={
        "": [".git", ".git/*", ".idea", ".gitignore", "doc/*", "atest/*"]
    },
    package_data={"TestToolsMK": ["VERSION"]},
    install_requires=[
        "robotframework>=5.0",
        "robotframework-databaselibrary",
        "robotframework-seleniumlibrary",
        "Pillow",
    ],
)
